﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Runtime.InteropServices.WindowsRuntime;
using Windows.ApplicationModel.Background;
using Windows.Devices.Geolocation;
using Windows.Devices.Geolocation.Geofencing;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.UI.Core;
using Windows.UI.Popups;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;

// The Blank Page item template is documented at http://go.microsoft.com/fwlink/?LinkId=234238

namespace App2
{
    /// <summary>
    /// An empty page that can be used on its own or navigated to within a Frame.
    /// </summary>
    public sealed partial class MainPage : Page
    {
        private ConstructorType _constType;

        public MainPage()
        {
            this.InitializeComponent();

            rbGeo1.IsChecked = true;
            InitUI();

            _constType = ConstructorType.Geo1;
            UpdateUI();

            GetGeofences();

            GeofenceMonitor.Current.GeofenceStateChanged += Current_GeofenceStateChanged;

            // Register background task
            RegisterBackgroundTask();

        }

        void Current_GeofenceStateChanged(GeofenceMonitor sender, object args)
        {
            Dispatcher.RunAsync(CoreDispatcherPriority.Normal, () =>
            {
                lvReports.ItemsSource = GeofenceMonitor.Current.ReadReports();
            });
        }

        private async void RegisterBackgroundTask()
        {
            BackgroundAccessStatus accessStatus = await BackgroundExecutionManager.RequestAccessAsync();

            if (accessStatus == BackgroundAccessStatus.AllowedMayUseActiveRealTimeConnectivity ||
               accessStatus == BackgroundAccessStatus.AllowedWithAlwaysOnRealTimeConnectivity)
            {

                BackgroundTaskBuilder taskBuilder = new BackgroundTaskBuilder();
                taskBuilder.Name = "GeoBackground";
                taskBuilder.TaskEntryPoint = "BackgroundWRC.GeoBackground";

                LocationTrigger trigger = new LocationTrigger(LocationTriggerType.Geofence);

                taskBuilder.SetTrigger(trigger);

                taskBuilder.Register();
            }

        }

        private async void InitUI()
        {
            tbName.Text = "";

            Geolocator geo = new Geolocator();
            Geoposition pos = await geo.GetGeopositionAsync();
            tbLat.Text = pos.Coordinate.Point.Position.Latitude.ToString();
            tbLong.Text = pos.Coordinate.Point.Position.Longitude.ToString();

            Geocircle shape = new Geocircle(pos.Coordinate.Point.Position, 50.0);

            tbRad.Text = "25";
            cbEntered.IsChecked = true;
            cbExited.IsChecked = true;
            cbRemoved.IsChecked = false;

            rbSingleYes.IsChecked = true;

            tbDwellTime.Text = "60";
            tbDuration.Text = "300";

        }

        private void GeoGroupChecked(object sender, RoutedEventArgs e)
        {
            switch ((sender as RadioButton).Name)
            {
                case "rbGeo1":
                    _constType = ConstructorType.Geo1;
                    break;
                case "rbGeo2":
                    _constType = ConstructorType.Geo2;
                    break;
                case "rbGeo3":
                    _constType = ConstructorType.Geo3;
                    break;
                case "rbGeo4":
                    _constType = ConstructorType.Geo4;
                    break;
            }

            UpdateUI();
        }

        private void UpdateUI()
        {
            spMonitor.Visibility = (_constType == ConstructorType.Geo1) ? Visibility.Collapsed : Visibility.Visible;

            spDwelltime.Visibility = (_constType == ConstructorType.Geo1 || _constType == ConstructorType.Geo2) ? Visibility.Collapsed : Visibility.Visible;

            spDuration.Visibility = (_constType == ConstructorType.Geo4) ? Visibility.Visible : Visibility.Collapsed;
          
        }
        public enum ConstructorType
        {
            Geo1,
            Geo2,
            Geo3,
            Geo4
        }

        private void CreateClicked(object sender, RoutedEventArgs e)
        {
            // unique id
            string name = tbName.Text;

            // geoshape
            BasicGeoposition pos = new BasicGeoposition() { Latitude = Convert.ToDouble(tbLat.Text),
                                                            Longitude = Convert.ToDouble(tbLong.Text)};
            Geocircle circle = new Geocircle(pos, Convert.ToDouble(tbRad.Text));

            Geofence geofence;

            if(_constType == ConstructorType.Geo1) {
                geofence = new Geofence(name, circle);
            } else {

                // Monitoring states
                MonitoredGeofenceStates states = MonitoredGeofenceStates.None;

                if(cbEntered.IsChecked.HasValue && cbExited.IsChecked.Value)
                    states |= MonitoredGeofenceStates.Entered;

                if(cbExited.IsChecked.HasValue && cbExited.IsChecked.Value)
                    states |= MonitoredGeofenceStates.Exited;

                if(cbRemoved.IsChecked.HasValue && cbRemoved.IsChecked.Value)
                    states |= MonitoredGeofenceStates.Removed;

                // Single Use
                bool isSingleUse = rbSingleYes.IsChecked.HasValue && rbSingleYes.IsChecked.Value;
                
                if( _constType == ConstructorType.Geo2) {
                    geofence = new Geofence(name, circle, states, isSingleUse);            
                } else {
                    // Dwell Time
                    TimeSpan dwellTime = new TimeSpan(0,0,Convert.ToInt32(tbDwellTime.Text));

                    if(_constType == ConstructorType.Geo3) {
                        geofence = new Geofence(name, circle, states, isSingleUse, dwellTime);
                    } else {
                        // State Time
                        DateTime start = DateTime.Now;

                        // Duration
                        TimeSpan duration = new TimeSpan(0,0,Convert.ToInt32(tbDuration.Text));
                        geofence = new Geofence(name, circle, states, isSingleUse, dwellTime, start, duration);
                    }

                }
              
            }

            if(geofence != null) {
                GeofenceMonitor.Current.Geofences.Add(geofence);

                var diag = new MessageDialog("Geofence added.");
                diag.ShowAsync();
            }
            else
            {
                var diag = new MessageDialog("Geofence was not created.");
                diag.ShowAsync();

            }

            InitUI();
            GetGeofences();
        }

        private async void GetGeofences()
        {

            lvFences.ItemsSource = GeofenceMonitor.Current.Geofences;

        }
    }
}
